//Windows header
#ifdef WIN32
#include <windows.h>
#endif

// OpenGL header
#include "gl.h"
#include "glut.h"


#include <iostream>
#include <stdlib.h>
using namespace std;

#include "tree.h"


#include <math.h>
#ifndef M_PI
	#define M_PI 3.14159265358979323846
#endif

GLuint	textureGrass, textureSky;

Tree* arbre;
int age;

typedef struct {
    unsigned long sizeX;
    unsigned long sizeY;
    char *data;
}Image;


int loadImage(char *filename, Image *image) {

    // make sure the file is there.
    FILE *file;		
    if ((file = fopen(filename, "rb"))==NULL) {
				cout << "File Not Found : " << filename << endl;
				return 0;
    }
    
    // seek through the bmp header, up to the width/height:
    fseek(file, 18, SEEK_CUR);

    // read width and height, and calculate the size of the image
    unsigned long i;
    if ((i = fread(&image->sizeX, 4, 1, file)) != 1) {
				cout << "Error reading width from " << filename << endl;
				return 0;
    }    
    if ((i = fread(&image->sizeY, 4, 1, file)) != 1) {
				cout << "Error reading height from " << filename << endl;
				return 0;
    }    
		unsigned long size = image->sizeX * image->sizeY * 3;

    // read the planes
    unsigned short int planes;
    if ((fread(&planes, 2, 1, file)) != 1) {
				cout << "Error reading planes from " << filename << endl;
				return 0;
    }
    if (planes != 1) {
				cout << "Planes from " << filename << "is not 1: " << planes << endl;
				return 0;
    }

    // read the bpp
    unsigned short int bpp;
    if ((i = fread(&bpp, 2, 1, file)) != 1) { 
				cout << "Error reading bpp from " << filename << endl;
				return 0;
    }
    if (bpp != 24) {
				cout << "Bpp from " << filename << " is not 24: " << bpp << endl;
				return 0;
    }
	
    // seek past the rest of the bitmap header.
    fseek(file, 24, SEEK_CUR);

    // read the data. 
    image->data = (char *) malloc(size);
    if (image->data == NULL) {
				cout << "Error allocating memory for color-corrected image data" << endl;
				return 0;	
    }

    if ((i = fread(image->data, size, 1, file)) != 1) {
				cout << "Error reading image data from " << filename << endl;
				return 0;
    }

		// reverse all of the colors. (bgr -> rgb) 
		char temp;                         
		for (i=0;i<size;i+=3) { 
				temp = image->data[i];
				image->data[i] = image->data[i+2];
				image->data[i+2] = temp;
    }
		
    return 1;
}

GLvoid loadGLTexture(char* textureName, GLuint* textureIdx) {	
    // Load Texture
    Image *image1;
    
    // allocate space for cube texture
    image1 = (Image *) malloc(sizeof(Image));
    if (image1 == NULL) {
				printf("Error allocating space for image");
				exit(0);
    }
    if (!loadImage(textureName, image1)) {
				exit(1);
    }        

    // Create Texture
    glGenTextures(1, textureIdx);
    glBindTexture(GL_TEXTURE_2D, *textureIdx);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR_MIPMAP_NEAREST); 
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT); 
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT); 
	glTexImage2D(GL_TEXTURE_2D, 0, 3, image1->sizeX, image1->sizeY, 0, GL_RGB, GL_UNSIGNED_BYTE, image1->data);
	gluBuild2DMipmaps(GL_TEXTURE_2D, 3, image1->sizeX, image1->sizeY, GL_RGB, GL_UNSIGNED_BYTE, image1->data);
	
};


////////////////////////////////////
//  Camera :
////////////////////////////////////
float cameraAngle=M_PI*0.5f;
float distToOrigin = 27.0;
float cameraHeight = 6.0f;
void setCamera() {
	float x = distToOrigin*cos(cameraAngle);
	float y = cameraHeight;
	float z = distToOrigin*sin(cameraAngle);
	glLoadIdentity();
	gluLookAt(x, y, z, -x, y, -z, 0.0f,1.0f,0.0f);
}

////////////////////////////////////
//  Dessin :
////////////////////////////////////
void drawSky() {
	glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);	// disable lighting effects on background

	glBindTexture(GL_TEXTURE_2D, textureSky);

	float z=300.0;
	float x=200;
	float y=100;
	glBegin(GL_QUADS);

	// Top :
	glTexCoord2f(0.0f, 0.0f); glVertex3f(-x, y, -z);
	glTexCoord2f(1.0f, 0.0f); glVertex3f(x, y, -z);
	glTexCoord2f(1.0f, 1.0f); glVertex3f(x,  y, z);
	glTexCoord2f(0.0f, 1.0f); glVertex3f(-x,  y, z);


	// Sides :
	glTexCoord2f(0.0f, 0.0f); glVertex3f(-x, -0.0f, -z);
	glTexCoord2f(1.0f, 0.0f); glVertex3f(x, -0.0f, -z);
	glTexCoord2f(1.0f, 1.0f); glVertex3f(x,  y, -z);
	glTexCoord2f(1.0f, 0.0f); glVertex3f(-x,  y, -z);

	glTexCoord2f(0.0f, 0.0f); glVertex3f(-x, -0.0f, z);
	glTexCoord2f(1.0f, 0.0f); glVertex3f(x, -0.0f, z);
	glTexCoord2f(1.0f, 1.0f); glVertex3f(x,  y, z);
	glTexCoord2f(0.0f, 1.0f); glVertex3f(-x,  y, z);

	glTexCoord2f(0.0f, 0.0f); glVertex3f(x, -0.0f, -z);
	glTexCoord2f(1.0f, 0.0f); glVertex3f(x, -0.0f, z);
	glTexCoord2f(1.0f, 1.0f); glVertex3f(x,  y, z);
	glTexCoord2f(0.0f, 1.0f); glVertex3f(x,  y, -z);

	glTexCoord2f(0.0f, 0.0f); glVertex3f(-x, -0.0f, -z);
	glTexCoord2f(1.0f, 0.0f); glVertex3f(-x, -0.0f, z);
	glTexCoord2f(1.0f, 1.0f); glVertex3f(-x,  y, z);
	glTexCoord2f(0.0f, 1.0f); glVertex3f(-x,  y, -z);
	

	glEnd();
}

void drawGrass() {
	glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);

	glBindTexture(GL_TEXTURE_2D, textureGrass);

	float z=300.0;
	float x=200;
	float y=0;
	glBegin(GL_QUADS);

	// The "grass" texture is drawn horizontally.
	glTexCoord2f(1.0f, 0.0f); glVertex3f(-x, y, z);
	glTexCoord2f(0.0f, 0.0f); glVertex3f(x, y, z);
	glTexCoord2f(0.0f, 1.0f); glVertex3f(x,  y, -z);
	glTexCoord2f(1.0f, 1.0f); glVertex3f(-x,  y, -z);

	glEnd();
}
void displayScene() {
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);


	drawSky();
	drawGrass();
	arbre->drawTree();
	age++;

	glutSwapBuffers();
}

void idleScene() {
}


////////////////////////////////////
//  Gestion d'vnements :
////////////////////////////////////
void specialKey(int key, int x_, int y_) {
	switch (key) {
        case GLUT_KEY_LEFT :
            cameraAngle += 0.1f;
            setCamera();  
			break;
        case GLUT_KEY_RIGHT :
			cameraAngle -=0.1f;
			setCamera();  
			break;
        case GLUT_KEY_UP :
            distToOrigin -= 0.2f;
            if (distToOrigin<1.5f) distToOrigin = 1.5f;
            setCamera();  
			break;
        case GLUT_KEY_DOWN :
            distToOrigin += 0.2f;
            setCamera();  
			break;
        case GLUT_KEY_PAGE_UP :
            cameraHeight += 0.1f;
            setCamera();  
			break;
        case GLUT_KEY_PAGE_DOWN :
            cameraHeight -= 0.1f;
            setCamera();  
			break;
	}
}

void commonKey(unsigned char key, int x_, int y_) {
	switch(key) {
		case 27 :
			glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT | GL_ACCUM_BUFFER_BIT);
			exit(0);
		default :
			return;
	}
}



void mouseClick(int button, int state, int x,int y) {
	
}


void mouseMotion(int x,int y) {

}

void changeSize(int w, int h)   {
	glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
	
	float ratio = 1.0f * w / h;
	glViewport(0, 0, w, h);
	gluPerspective(45,ratio,1,1000);
	glMatrixMode(GL_MODELVIEW);
	setCamera();
}


////////////////////////////////////
//  Mthodes d'initialisation :
////////////////////////////////////

// Initialisation :
void initScene(void) {
	// init GL states
	glClearDepth(1.0);	
	glClearColor(1.0, 1.0, 1.0, 1.0);							

	glDepthFunc(GL_LESS);
	glEnable(GL_DEPTH_TEST);

	glShadeModel(GL_SMOOTH);
	glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);

	GLfloat lightAmbient[]= { 0.2f, 0.2f, 0.2f, 1.0f };
	GLfloat lightDiffuse[]= { 1.0f, 1.0f, 1.0f, 1.0f };
	float lightAngle = 0.0f;
	GLfloat lightPosition[]= { 4.0*cos(lightAngle), 3.0f, 4.0*sin(lightAngle), 1.0f };
    glLightfv(GL_LIGHT0, GL_AMBIENT, lightAmbient);
    glLightfv(GL_LIGHT0, GL_DIFFUSE, lightDiffuse);
	glLightfv(GL_LIGHT0, GL_POSITION,lightPosition);
	glEnable(GL_LIGHT0);
	glEnable(GL_LIGHTING);

	glEnable(GL_COLOR_MATERIAL);

	// texture
	loadGLTexture("textureSky.bmp", &textureSky);
	loadGLTexture("textureGrass.bmp", &textureGrass);

	arbre = new Tree(18.0,1.0);
	age = 0;
	loadGLTexture("textureTronc.bmp", &(arbre->textureTronc));

	glEnable(GL_TEXTURE_2D);
}

////////////////////////////////////
//  Main method
////////////////////////////////////
int main(int argc, char **argv) {
	// crer la fentre
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGBA);
	glutInitWindowPosition(100,100);
	glutInitWindowSize(800, 600);
	glutCreateWindow("Dynamics");

	// Evnements clavier/souris :
	glutSpecialFunc(specialKey);
	glutKeyboardFunc(commonKey);
	glutMouseFunc(mouseClick);
	glutMotionFunc(mouseMotion);
	
	// Mthodes de rendu :
	glutDisplayFunc(displayScene);
	glutIdleFunc(displayScene);

	// Changement de taille de la fentre :
	glutReshapeFunc(changeSize);

	// Initialise OpenGL :
	initScene();

	// run program
	glutMainLoop();

	return(0);
}
